package ucl.salinization.model.landuse;

import java.util.HashMap;
import java.util.Map;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.PrecisionModel;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.random.RandomHelper;
import ucl.salinization.model.physical.Field;
import ucl.salinization.model.physical.SalinizationModel;
import ucl.salinization.social.Person;


/**
 * Model on land use behaviors for agriculture affecting salinization.
 * 
 * @author Mark Altaweel
 *
 */
public class LandUse {
	
	private Field field;
	Map<Integer,Integer> yearDone=new HashMap<Integer,Integer>();
	Map<Integer,Integer> saltYear=new HashMap<Integer,Integer>();
	Map<Integer,Integer> fYear=new HashMap<Integer,Integer>();
	Map<Field,Integer> fields=new HashMap<Field,Integer>();
	
	private Person person;
	boolean move=true;

	/**
	 * Main object constructor.
	 * @param p the person cropping a field.
	 * @param field the field to be cropped.
	 */
	public LandUse (Person p,Field field){
		this.person=p;
		this.field=field;
	}
	
	/**
	 * Check to see what day is it in a year.
	 * @return a day value.
	 */
	public double dayInYear(){
		double tick=RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		double daysPT=(Double)RunEnvironment.getInstance().getParameters().getValue("daysPerTick");
		
		double dayInYear=(tick*daysPT)%365.25d;
		
		return dayInYear;
	}
	
	/**
	 * Check to see what year is it.
	 * @return a year value.
	 */
	public int checkYear(){
		double tick=RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		double daysPT=(Double)RunEnvironment.getInstance().getParameters().getValue("daysPerTick");
	
		int year=(int) Math.round(tick*daysPT/365.25);
		
		return year;
	}
	
	/**
	 * Check to see if irrigation part of the year.
	 * @return true or false.
	 */
	public boolean checkIrrigationTime(){
		boolean timeToIrrigate=false;
			
		int year=checkYear();
		
		double dayInYear=dayInYear();
		if(dayInYear<153){
			if(!yearDone.containsKey(year)){
				yearDone.put(year, year);
				timeToIrrigate=true;
			}
			else{
				return false;
			}
		}
		return timeToIrrigate;
	}
	
	/**
	 * Check to see if it is planting season.
	 * @return true or false.
	 */
	public boolean checkPlantingTime(){
		boolean planting=false;
		double dayInYear=dayInYear();
		if(dayInYear> 305){
			planting=true;
		}
		return planting;
	}
	
	/**
	 * Main method to launch agricultural process.
	 */
	public void step(){
		SalinizationModel sm=(SalinizationModel) RunState.getInstance().getMasterContext().
			getObjects(SalinizationModel.class).iterator().next();
		
//		if(!saltYear.containsKey(checkYear())){
//			saltYear.put(checkYear(),checkYear());
//		}
		
		if(fallowField()){
			this.person.moveTo(this.field.getCommunity().getLocation());
			field.setStatus(Field.FALLOW_FIELD);
			if(!saltYear.containsKey(checkYear())){
				sm.drainSalt(field);
				saltYear.put(checkYear(),checkYear());
			}
			
		}
		else{
			field.setTrueFallowYears(0.0);
			field.setStatus(Field.IRRIGATION);
			if(!saltYear.containsKey(checkYear())){
				sm.capilaryRise(field);
				saltYear.put(checkYear(),checkYear());
			}
			cropField();
			field.setInitialSalinization(field.getSaltCompounds());
		}
		/**
		 * TODO cap rise and drain salt yearly
		 */
//		sm.drainSalt(field);
//		sm.capilaryRise(field);
	}
	
	/**
	 * Method to crop a field during a year.
	 * Agent also returns to village at periodic intervals.
	 */
	public void cropField(){
		double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		if(checkPlantingTime()){
			
			if(move && tick>8){
				
				this.person.moveTo(field.getCentroid());
		//		System.out.println("move to field");
				move=false;
			}
			else if(tick>8){	
				double d=RandomHelper.nextDouble();
				if(d>.5){
					double xx=this.person.getCommunity().getLocation().getX();
					double yy=this.person.getCommunity().getLocation().getY();
					double x2=this.field.getPoly().getCentroid().getX();
					double y2=this.field.getPoly().getCentroid().getY();
					double newX=(xx+x2)/2;
					double newY=(yy+y2)/2;
					PrecisionModel pm=this.person.getCommunity().getLocation().getPrecisionModel();
					int srid=this.person.getCommunity().getLocation().getSRID();
					this.person.moveTo(new Point(new Coordinate(newX,newY),pm,srid));
				}
				else
					this.person.moveTo(this.field.getCommunity().getLocation());
		//		System.out.println("move to village");
				move=true;
				
			}
			else
				this.person.moveTo(this.field.getCommunity().getLocation());
			field.setStatus(Field.PLANTED);
		}
		else{
			if(checkIrrigationTime() && tick>8){
				field.setStatus(Field.IRRIGATION);
				this.person.moveTo(this.field.getCentroid());
				irrigate();
			}
		}
	}
	
	/**
	 * Check if the field should be left fallow.
	 * @return boolean for fallow check.
	 */
	public boolean fallowField(){
		boolean fallow=false;
		int fallowYear=(Integer)RunEnvironment.getInstance().getParameters().getValue("fallowYear");
		
		int year=checkYear();
		if(field.getFallowYears()>=fallowYear-1.0){
			fallow=true;
			if(!fYear.containsKey(year)){
				field.setFallowYears(field.getFallowYears()-1.0);
				fYear.put(year,year);
			}
			return fallow;
		}
	
		if(field.getPlantYear()%fallowYear!=0)
			fallow=true;
		
		return fallow;
	}
	
	/**
	 * Method to irrigate the field
	 * @param f a field object
	 */
	public void irrigate(){
		SalinizationModel sm=(SalinizationModel) RunState.getInstance().getMasterContext().
			getObjects(SalinizationModel.class).iterator().next();
		
		sm.run(field);
		
		//adjust years to leave a field fallow based on salinization
		if(sm.getSaltTolerance() >= field.getYield() && field.getFallowYears()<1.0){
			field.setTrueFallowYears(1+Math.rint(Math.pow(((sm.getSaltTolerance()-field.getYield())*sm.getFallowScaling()),2)));
			field.setFallowYears(1+Math.rint(Math.pow(((sm.getSaltTolerance()-field.getYield())*sm.getFallowScaling()),2)));
		}
	}
}
